// CalendarGraphics.cpp by T. Yamagata
//	Last Update: 981215

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <app/Looper.h>							//981205
#include <kernel/OS.h>							//981205
#include <interface/Bitmap.h>					//981215

#include "CalendarGraphics.h"					//981124
#include "StringLabel.h"
#include "YYMMDD.h"								//981114

#include <View.h>

//=======================================================

static char *Singledaynames[] =
{
	"S",
	"M",
	"T",
	"W",
	"T",
	"F",
	"S"
};

#ifdef CALJ											//981114
static char *Shortdaynames[] =
{
	"日",
	"月",
	"火",
	"水",
	"木",
	"金",
	"土"
};

static char *Daynames[] =
{
	"日曜日",
	"月曜日",
	"火曜日",
	"水曜日",
	"木曜日",
	"金曜日",
	"土曜日"
};
#else												//981114
#define Shortdaynames dowList

static char *Daynames[] =
{
	"Sunday",
	"Monday",
	"Tuesday",
	"Wednesday",
	"Thursday",
	"Friday",
	"Saturday"
};

static char *Shortmonthnames[] =
{"",
	"Jan",
	"Feb",
	"Mar",
	"Apr",
	"May",
	"Jun",
	"Jul",
	"Aug",
	"Sep",
	"Oct",
	"Nov",
	"Dec"};

static char *Monthnames[] =
{	"",
	"January",
	"February",
	"March",
	"April",
	"May",
	"June",
	"July",
	"August",
	"September",
	"October",
	"November",
	"December"};
#endif															//981114

static int holidays[12][3] = {
	{ 0,14,-2}, {10,-2,-2}, {19,-2,-2},{28,-2,-2},
	{ 2, 3, 4}, {-2,-2,-2}, {19,-2,-2},{-2,-2,-2},
	{14,22,-2}, { 9,-2,-2}, { 2,22,-2},{22,-2,-2}};
	
float sizeThresh[4] =											//981203
	{100.0, 140.0, 378.0, 500.0 };								//981203

float monthFontSizeList[5] =									//981203
	{10.0, 12.0, 14.0, 18.0, 24.0 };							//981203
	
float daysFontSizeList[5] =										//981203
	{ 8.0,  9.0, 10.0, 12.0, 18.0 };							//981203

int32
GetSizeIndex( float width )
{
	int32	sizeIndex;
	
	if (width < 100.0)
	{
		sizeIndex = 100;
	} else if (width < 140.0)
	{
		sizeIndex = 140;
	} else if (width < 378.0)
	{
		sizeIndex = 378;
	}else if (width < 500.0)
	{
		sizeIndex = 500;
	}else
	{
		sizeIndex = 1000;
	}
	return sizeIndex;
}

//.............................................................
// TimerView
//.............................................................
void TimerView::FrameResized(float new_width, float new_height)
{
	SetSize(BPoint(new_width, new_height));
}

void TimerView::SetSize(BPoint size)
{
	fSize = size;
	Invalidate();
}

void TimerView::Invalidate()
{	
	if (!p->thisYearMonth)
		CalculateMonthLabels();
	Draw(Bounds());												//981209
}

void
TimerView::Draw(BRect updateRect)								//981203
{
	SetHighColor(p->viewColor);									//981209
	FillRect(updateRect);										//981209
	SetHighColor(0,0,0);										//981209																//981203
	if (p->thisYearMonth) {
		time( &ltime );
		strcpy( timebuf, ctime( &ltime ) );
    	char datestring[64]={0};
#ifdef CALJ														//981114
		getToday();
		sprintf(datestring,"%ld年%ld月%d日 (%s) %8.8s",p->year,p->month,
			atoi(timebuf+8),Shortdaynames[dayOfWeek],timebuf+11);
#else
		strcpy(datestring, timebuf);
		datestring[24] = '\0';			//remove lf code		//981209
#endif
		SetFontSize(p->monthFontSize);
		DrawString(datestring, BPoint(4.0, p->monthFontSize+4.0));
	} else
		fdateLabel->Draw(this);
}																//981203

TimerView::TimerView(BRect frame, char *name, 
	CalendarView *parent)									//981203
	: BView(frame, name, B_FOLLOW_ALL,						//981203
    B_FULL_UPDATE_ON_RESIZE|B_WILL_DRAW|B_FRAME_EVENTS|B_PULSE_NEEDED),//981203
         p(parent)											//981203
{															//981203
	fdateLabel = new StringLabel ("",BPoint(14.0,			//981209
		p->monthFontSize + p->GridMargin),p->monthFontSize);//981209
	Invalidate();
}

void
TimerView::Pulse()
{
	((BLooper *)Window())->Lock();
	Draw(Bounds());
	Sync();
	((BLooper *)Window())->Unlock();
}	
void
TimerView::CalculateMonthLabels()
{
    char datestring[64]={0};
#ifdef CALJ													//981114
	sprintf(datestring,"%ld 年 -  %ld 月", p->year,  p->month);
#else														//981209
	char **monthnameArray;
	// determine the array to use for weekday names based on the width
	// of the columns.
	if (p->GridCellSize.x < 20.0)
	{
		monthnameArray = Shortmonthnames;
	} else if (p->GridCellSize.x < 54.0)
	{
		monthnameArray = Shortmonthnames;
	} else
	{
		monthnameArray = Monthnames;
	}
	// Create and assign a name for the month label
    char *monthname = monthnameArray[p->month];				//981209
	sprintf(datestring,"%s -  %ld",monthname, p->year);		//981209
#endif														//981114

	fdateLabel->SetText(datestring);
    fdateLabel->SetOrigin(BPoint (14.0,p->monthFontSize + 
    	p->GridMargin));
	fdateLabel->SetSize(p->monthFontSize);
}

//...................................................................
// DaysView
//...................................................................
DaysView:: DaysView(BRect frame, char *name, CalendarView *parent)
	: BView(frame, name, B_FOLLOW_ALL,						//981203
         B_FULL_UPDATE_ON_RESIZE|B_WILL_DRAW|B_FRAME_EVENTS),//981203
    p(parent),												//981205
	fSize(100,100),	
	fSizeIndex(0),
	fColumn0Label(0),
	fColumn1Label(0),
	fColumn2Label(0),
	fColumn3Label(0),
	fColumn4Label(0),
	fColumn5Label(0),
	fColumn6Label(0)
{															//981203
	Init();													//981205
	SetViewColor(B_TRANSPARENT_32_BIT);						//981207
}															//981203

void
DaysView::Init()
{
	GridTopLeft = BPoint(11.0, 43.0);
	p->GridCellSize = BPoint(34.0,28.0);
	GridBottomRight = BPoint(GridTopLeft.x+(7*p->GridCellSize.x),
				GridTopLeft.y+(6*p->GridCellSize.y));
	GridRect = BRect(GridTopLeft, GridBottomRight);	

	// Create the day labels for each column
	fColumn0Label = CreateDayLabel(0);
	fColumn1Label = CreateDayLabel(1);
	fColumn2Label = CreateDayLabel(2);
	fColumn3Label = CreateDayLabel(3);
	fColumn4Label = CreateDayLabel(4);
	fColumn5Label = CreateDayLabel(5);
	fColumn6Label = CreateDayLabel(6);
	
	// Fill in the initial text array
	for (long counter=0; counter < 32; counter++)
	{
		char daystring[12]={0};
		sprintf(daystring,"%ld",counter);					//981030
		
		GDayTextArray[counter] = 
		new StringLabel(daystring,BPoint(0,0),p->daysFontSize);
	}
	
	// Set the initial size
	BPoint boundsSize(Bounds().Width(),Bounds().Height());
	SetSize(boundsSize);
}

void DaysView::FrameResized(float new_width, float new_height)
{
	SetSize(BPoint(new_width, new_height));
}

void DaysView::SetSize(BPoint size)
{
	fSize = size;
	Invalidate();
}

void DaysView::Invalidate()
{	
	// Do the resizing magic
	// Re-calculate the various size things
	CalculateParameters();
	CalculateDaysLabels();
	
	// Make sure the view draws itself
	Draw(Bounds());										//981209	
}


void DaysView::CalibrateFontSizes(const long sizeIndex)
{
	// If the new index is the same as the old, then do nothing
	if (sizeIndex == fSizeIndex)
		return;
	p->daysFontSize = GetFontSize( sizeIndex, sizeThresh,
		daysFontSizeList );		

	// Set the point size of the number displays
	for (long counter=0; counter < 32; counter++)
	{
		GDayTextArray[counter]->SetSize(p->daysFontSize);
	}
	
	fSizeIndex = sizeIndex;
}

void
DaysView::CalculateFontSizes()
{
	BRect bounds = Bounds();
	long sizeIndex = GetSizeIndex( bounds.Width() );

	CalibrateFontSizes(sizeIndex);
}

void
DaysView::CalculateParameters()
{
	// Do the calculation of the month stuff first
	// because everything else depends on it.
	CalculateFontSizes();

	// Re-Calculate the grid sizes and whatnot based on the clipped
	// size of the view.  Thus we can shrink the display to always
	// fit within the confines of the displayed area.
	BRect bounds = Bounds();
	double margin = p->GridMargin;

	GridTopLeft = BPoint(margin, p->daysFontSize+margin);
	p->GridCellSize= BPoint((bounds.Width()-margin*2)/7,
					((bounds.Height()-GridTopLeft.y)/6));
	GridBottomRight = BPoint(GridTopLeft.x+(7*p->GridCellSize.x),
					GridTopLeft.y+(6*p->GridCellSize.y));
	GridRect	= BRect(GridTopLeft, GridBottomRight);
}

void
DaysView::CalculateColumnLabel(StringLabel &theLabel, int32 theColumn)
{
	// Calculate an origin such that the text will be centered
	BPoint cellSize = p->GridCellSize;
	BPoint center( GridTopLeft.x+theColumn*cellSize.x + (cellSize.x/2),
		GridTopLeft.y-3.0);
	
	theLabel.SetText(fDayNameArray[theColumn]);
    theLabel.SetOrigin(center);
	theLabel.SetSize(p->daysFontSize);
	theLabel.SetAlignment(B_ALIGN_CENTER);
}

void
DaysView::CalculateDaysLabels()
{
	// determine the array to use for weekday names based on the width
	// of the columns.
	if (p->GridCellSize.x < 20.0)
	{
		fDayNameArray = Singledaynames;
	} else if (p->GridCellSize.x < 54.0)
	{
		fDayNameArray = Shortdaynames;
	} else
	{
		fDayNameArray = Daynames;
	}

	// Now Setup the day headings
	CalculateColumnLabel(*fColumn0Label, 0);
	CalculateColumnLabel(*fColumn1Label, 1);
	CalculateColumnLabel(*fColumn2Label, 2);
	CalculateColumnLabel(*fColumn3Label, 3);
	CalculateColumnLabel(*fColumn4Label, 4);
	CalculateColumnLabel(*fColumn5Label, 5);
	CalculateColumnLabel(*fColumn6Label, 6);
}


StringLabel *
DaysView::CreateDayLabel(const int32 column)
{
	char **daynameArray;

	// determine the array to use for weekday names based on the width
	// of the columns.
	// If they are less than 58, then use the short names
	if (p->GridCellSize.x < 54.0)
	    daynameArray = (char **)Shortdaynames;				//981114
	else
		daynameArray = Daynames;

	StringLabel *newLabel = new StringLabel(daynameArray[column]);
	//newLabel->SetColor(TTextColorStyle::GetBlack());
    newLabel->SetOrigin(BPoint (column*p->GridCellSize.x +
    	GridTopLeft.x,40.0));
	newLabel->SetSize(p->daysFontSize);

	return newLabel;
}


//=====================================================================
//	Method: Draw
//=====================================================================
void
DaysView::Draw(BRect updateRect)
{
	BPoint cellSize = p->GridCellSize;						//981205

	SetHighColor(p->viewColor);	
	FillRect(updateRect);									//981209
	
	// Draw the outline that will hold the grid
	SetHighColor(255,255,255);	
	FillRect(GridRect);
	SetHighColor(0,0,0);	
	StrokeRect(GridRect);
	
	// If the Calendar is for this month, indicate today by yellow
	// background											//981114
	if (p->thisYearMonth) {									//981122
		getToday();											//981122
		p->month = thisMonth;								//981209
		p->year = thisYear;									//981209
		SetHighColor(255,255,0);							//981114
		//get the row of this week							//981209
		int32 row = (day-dayOfWeek+5)/7;					//981209
		BRect cell(2.0,1.0,cellSize.x,cellSize.y-1);		//981114
		cell.OffsetBy( cellSize.x * dayOfWeek + 			//981114
		GridTopLeft.x-1,cellSize.y*row+GridTopLeft.y);		//981114
		FillRect( cell );									//981114
		SetHighColor(0,0,0);								//981114
	}														//981114
		
	// Draw the day labels
	fColumn0Label->Draw(this);
	fColumn1Label->Draw(this);
	fColumn2Label->Draw(this);
	fColumn3Label->Draw(this);
	fColumn4Label->Draw(this);
	fColumn5Label->Draw(this);
	fColumn6Label->Draw(this);
	
	// Draw the vertical lines in the grid.
	float xStart = GridTopLeft.x+1.0;
	float xEnd = GridBottomRight.x-1.0;
	float yStart = GridTopLeft.y+1.0;
	float yEnd = GridBottomRight.y-1.0;
	int counter = 0;

	
	// If the cell height is greater than 15.0 then draw the vertical
	// lines, otherwise they just get in the way.
	if (cellSize.y > 15.0)
	{
		for (counter = 1; counter < 7; counter++)
		{
			BPoint startPoint(xStart+(counter*cellSize.x)-2,yStart);
			BPoint endPoint(xStart+(counter*cellSize.x)-2,yEnd);
			StrokeLine(startPoint,endPoint);
		}
	}
	
	// Draw the 5 horizontal lines in the grid.
	for (counter = 1; counter < 6; counter++)
	{	    
		BPoint startPoint(xStart,yStart+(counter*cellSize.y));
		BPoint endPoint(xEnd,yStart+(counter*cellSize.y));
		StrokeLine(startPoint,endPoint);
	}

	// NOW DRAW EVERYTHING ELSE	
	// Get the day of the week of the first day in the month
	//int mm = month;
	int dow = getDayOfWeek( p->year, p->month, 1);				//981209
	
	// Get the number of days in the month
	int maxdays = daysInMonth( p->year, p->month );				//981209
	
    int currentRow = 0;

	// Draw the values of the current selection
    int dayctr = 0, holiday1 = 0, holiday2 = 0, holiday3 = 0;	//981105
	int tempDow = dow;

	// Holidays of the month
	if (p->showHolidays) {
		holiday1 = holidays[p->month-1][0];
		holiday2 = holidays[p->month-1][1];
		holiday3 = holidays[p->month-1][2];
	}
	
	// Here we go through each day of the month and find out what cell it should
	// be displayed in and put a number in the center of the cell
	for (dayctr = 0; dayctr < maxdays; dayctr++)
	{
		float columnX = (tempDow+0.5) * cellSize.x + GridTopLeft.x;	//981209
		float rowY = currentRow * cellSize.y + GridTopLeft.y + 		//981209 
				+ (cellSize.y + p->daysFontSize)/2;					//981209
		
		// If the day is a weekday, droaw it in black.
		rgb_color dayColor = {0,0,0,0};
		
		// If it is Sunday, draw it in red.
		if (tempDow == 0) 
				dayColor.red  =  255;
		// If it's Saturday, draw it in blue.
		if (tempDow == 6) 
			dayColor.blue = 255;
		
		// If Japanese National Holidays are to be shown:	
		if (p->showHolidays) {
			if (dayctr==holiday1||dayctr==holiday2||dayctr==holiday3) {
				dayColor.red = 255;
				dayColor.blue = 0;
			}
			// If it is Monday, and the previous day is a National Holiday
			//    draw it in red.
			if (tempDow == 1)  {
				int	prevday = dayctr - 1;
				if (prevday==holiday1||prevday==holiday2||prevday==holiday3) 
					dayColor.red = 255;
			}
		}
				
		GDayTextArray[dayctr+1]->SetColor(dayColor);
		GDayTextArray[dayctr+1]->SetAlignment(B_ALIGN_CENTER);
		GDayTextArray[dayctr+1]->SetOrigin(BPoint(columnX,rowY));
		GDayTextArray[dayctr+1]->Draw(this);
		
	    // Increment the day of the week, if it is already  saturday == 6,
		// reset it to sunday == 0
	    if (tempDow == 6)
		{
			tempDow = 0;
			currentRow++;
		}else
		    tempDow++;
	}
	fImageNeedsUpdate = 0;
}

float														//981203
GetFontSize(float width, float thresh[4], float sizeList[5] )//981203
{															//981203
	int i;													//981203
	for (i = 0; i < 4; i++ )								//981203
		if (width < thresh[i]) break;						//981203
	return sizeList[i];										//981203
}															//981203

