// CalendarView.cpp by T. Yamagata
//	Last Update: 981217

#include <stdio.h>
#define	_BUILD_CALENDAR_VIEW_								//981125
#include <interface/Alert.h>								//981030
#include "CalendarView.h"									//981124
#include "CalendarGraphics.h"								//981124
#include "YYMMDD.h"											//981122

#include <Window.h>

//==================================================================
// Constructor
//==================================================================
CalendarView::CalendarView(BRect frame, char *name, int32 m, int32 y) 
	: BView(frame, name, B_FOLLOW_ALL,
         B_FULL_UPDATE_ON_RESIZE|B_WILL_DRAW|B_FRAME_EVENTS)//981124
{
	SetFont(be_bold_font);
	BFont	font; GetFont(&font);
	font.GetFamilyAndStyle(&family, &style);
	thisYearMonth = thisYear == y && thisMonth == m;		//981122
	rgb_color vc = {128,255,128,0};							//981215
	viewColor = vc;											//981215	
#ifdef CALJ													//981124
	InitCalendarView(m, y, true, thisYearMonth);			//981124
#else														//981124
	InitCalendarView(m, y, false, thisYearMonth);			//981124
#endif														//981124

}

//===================================================================
// Replicant Constructor
//===================================================================
CalendarView::CalendarView(BMessage *data)					//981224
	:BView(data)
{
	strcpy((char *)family, data->FindString("family"));
	strcpy((char *)style, data->FindString("style"));
	uint8	vc_red; data->FindInt8("vc_red", (int8 *)&vc_red);//981215
	uint8	vc_green;data->FindInt8("vc_green",(int8 *)&vc_green);//981215
	uint8	vc_blue; data->FindInt8("vc_blue",(int8 *)&vc_blue);//981215
	rgb_color	vc = {vc_red, vc_green, vc_blue, 0};		//981215
	viewColor = vc;											//981215

	InitCalendarView(data->FindInt32("month"),				//981124
				data->FindInt32("year"),
				data->FindBool("showHolidays"),
				data->FindBool("thisYearMonth"));			//981122
}

//===================================================================
//   MonthView Initialization
//===================================================================
void
CalendarView::InitCalendarView(int32 m, int32 y, bool s, bool tym)
															//981124
{
	if (tym) {												//981122
		getToday();											//981122
		month = thisMonth;									//981122
		year = thisYear;									//981122
	} else {												//981122
		month = m;
		year = y;
	}														//981122
	showHolidays = s;
	thisYearMonth = tym;
	
	GridMargin = 4.0;										//981203
	BRect	rect = Bounds();								//981203
	float	topViewSize;									//981203 
	SetViewColor( viewColor );								//981215
	monthFontSize = GetFontSize( rect.Width(), 				//981203
		sizeThresh, monthFontSizeList );					//981203
	float GridMargin = 4.0;									//981203
	rect.bottom = topViewSize = monthFontSize + GridMargin*2;//981205
	topView = new TimerView(rect, "Timer", this);			//981203
	AddChild( topView );									//981203
	rect = Bounds();										//981203
	rect.top = topViewSize;									//981203
	rect.bottom -= 7.0;										//981210
	bottomView = new DaysView(rect, "Days", this);			//981203
	AddChild( bottomView );									//981203
	BFont	font; GetFont(&font);							//981215
	font.SetFamilyAndStyle(family, style);
	topView->SetFont(&font);								//981215
	bottomView->SetFont(&font);								//981215
}															//981203
	
//===================================================================
// Method: Destructor
//===================================================================

CalendarView::~CalendarView()
{
}


//===================================================================
//	Method: FrameResized
//===================================================================

void		
CalendarView::FrameResized(float new_width, float new_height)	//981203
{                                                 				//981203
	monthFontSize = 											//981203
		GetFontSize(new_width, sizeThresh, monthFontSizeList);	//981203
	daysFontSize =												//981206
		GetFontSize(new_width, sizeThresh, daysFontSizeList);	//981206
	topView->ResizeTo(new_width, monthFontSize+2*GridMargin);	//981203
	bottomView->MoveTo(0.0, monthFontSize+2*GridMargin);		//981203
	bottomView->ResizeTo(new_width, 							//981206
		new_height-monthFontSize-2*GridMargin-7.0);				//981210
}																//981203

//===================================================================
// Method: Invalidate
//===================================================================
void
CalendarView::Invalidate()										//981208
{																//981208
	Window()->SetPulseRate(CheckThisYearMonth()? 1000000: 0);	//981209
	SetViewColor( viewColor );									//981215
	SetHighColor( viewColor );									//981215
	BRect	strip = Bounds();									//981215
	strip.top = strip.bottom - 7.0;								//981215
	strip.left = 7.0;											//981215
	FillRect(strip);											//981215
	topView->Invalidate();										//981208
	bottomView->Invalidate();									//981208
}
	
//===================================================================
// Method: Draw
//===================================================================

void
CalendarView::Draw(BRect updateRect)
{
	topView->Draw(topView->Bounds());							//981203
	bottomView->Draw(bottomView->Bounds());						//981203
}																//981203

//===================================================================
// Archive(): store data needed to replicate class
//===================================================================
extern const char *app_signature;
status_t
CalendarView::Archive(BMessage *data, bool deep) const
{
	BView::Archive(data, deep);
	data->RemoveName("_resize_mode");
	data->AddInt32("_resize_mode", B_FOLLOW_NONE);
#ifdef CALJ													//981126
	data->AddString("class", "CalendarViewJ");				//981126
#else														//981126
	data->AddString("class", "CalendarView");
#endif														//981126
	data->AddString("add_on", app_signature);
	data->AddInt32("month", month);
	data->AddInt32("year", year);
	data->AddBool("showHolidays", showHolidays);
	data->AddString("family", family);
	data->AddString("style", style);
	data->AddBool("thisYearMonth", thisYearMonth);			//981122
	data->AddInt8("vc_red", viewColor.red);					//981215
	data->AddInt8("vc_green", viewColor.green);				//981215
	data->AddInt8("vc_blue", viewColor.blue);				//981215
	return B_OK;
}

//===================================================================
// Instantiate(): Instantiate  the Calendar replicant
//===================================================================
CalendarView
*CalendarView::Instantiate(BMessage *data)
{
#ifdef CALJ													//981126
	if (validate_instantiation(data, "CalendarViewJ"))		//981126
#else														//981126
	if (validate_instantiation(data, "CalendarView"))
#endif														//981126
		return new CalendarView(data);
	// In case validation failed...
	return NULL;
}

void
CalendarView::MessageReceived(BMessage *a_msg)
{
	switch (a_msg->what) {
	case  B_ABOUT_REQUESTED:
		AboutRequested();
		break;
	default:
		BView::MessageReceived(a_msg);
		break;
	}
}

void
CalendarView::AboutRequested()
{
	const char msg[] =										//981022
#ifdef CALJ													//981114
			"CalendarJ\n"									//981114
#else														//981114
			"Calendar\n"									//981022
#endif														//981114
            "Version 2 for BeOS R4\n\n"						//981217
			"By T.Yamagata (yamagata@sie.co.jp)\n"			//981022
			"S I Electronics Ltd., Tokyo, Japan\n\n"		//981022
			"Modified from ftp.be.com/pub/Samples/January";	//981022

	BAlert * alert  = new BAlert( NULL, msg, "OK" );		//981022
	alert->Go();
}

bool
CalendarView::CheckThisYearMonth()							//981124
{															//981124
	thisYearMonth = year==thisYear && month==thisMonth;		//981124
	return thisYearMonth;									//981209
}															//981124


